@@ -27,16 +27,41 @@
 
 G_BEGIN_DECLS
 
+typedef struct _GMimeHeader GMimeHeader;
+
+
+/**
+ * GMimeHeaderList:
+ *
+ * A message or mime-part header.
+ **/
+typedef struct _GMimeHeaderList GMimeHeaderList;
+
+
 /**
  * GMimeHeaderIter:
+ * @hdrlist: a #GMimeHeaderList
+ * @cursor: a #GMimeHeader
+ * @version: the version of @hdrlist when initialized
  *
- * A message or mime-part header iterator.
+ * A message or mime-part header iterator. All members should be
+ * considered private.
  **/
 typedef struct _GMimeHeaderIter GMimeHeaderIter;
 
-GMimeHeaderIter *g_mime_header_iter_copy (GMimeHeaderIter *iter);
+struct _GMimeHeaderIter {
+	GMimeHeaderList *hdrlist;
+	GMimeHeader *cursor;
+	guint32 version;
+};
+
+
+GMimeHeaderIter *g_mime_header_iter_new (void);
 void g_mime_header_iter_free (GMimeHeaderIter *iter);
 
+GMimeHeaderIter *g_mime_header_iter_copy (GMimeHeaderIter *iter);
+void g_mime_header_iter_copy_to (GMimeHeaderIter *src, GMimeHeaderIter *dest);
+
 gboolean g_mime_header_iter_equal (GMimeHeaderIter *iter1, GMimeHeaderIter *iter2);
 
 gboolean g_mime_header_iter_is_valid (GMimeHeaderIter *iter);
@@ -54,14 +79,6 @@ const char *g_mime_header_iter_get_value (GMimeHeaderIter *iter);
 gboolean g_mime_header_iter_remove (GMimeHeaderIter *iter);
 
 
-/**
- * GMimeHeaderList:
- *
- * A message or mime-part header.
- **/
-typedef struct _GMimeHeaderList GMimeHeaderList;
-
-
 /**
  * GMimeHeaderWriter:
  * @stream: The output stream.
@@ -86,7 +103,7 @@ void g_mime_header_list_set (GMimeHeaderList *headers, const char *name, const c
 const char *g_mime_header_list_get (const GMimeHeaderList *headers, const char *name);
 gboolean g_mime_header_list_remove (GMimeHeaderList *headers, const char *name);
 
-GMimeHeaderIter *g_mime_header_list_get_iter (GMimeHeaderList *headers);
+gboolean g_mime_header_list_get_iter (GMimeHeaderList *headers, GMimeHeaderIter *iter);
 
 void g_mime_header_list_register_writer (GMimeHeaderList *headers, const char *name, GMimeHeaderWriter writer);
 ssize_t g_mime_header_list_write_to_stream (const GMimeHeaderList *headers, GMimeStream *stream);
