@@ -47,7 +47,6 @@ static void g_mime_message_part_init (GMimeMessagePart *message_part, GMimeMessa
 static void g_mime_message_part_finalize (GObject *object);
 
 /* GMimeObject class methods */
-static void message_part_init (GMimeObject *object);
 static void message_part_prepend_header (GMimeObject *object, const char *header, const char *value);
 static void message_part_append_header (GMimeObject *object, const char *header, const char *value);
 static void message_part_set_header (GMimeObject *object, const char *header, const char *value);
@@ -96,7 +95,6 @@ g_mime_message_part_class_init (GMimeMessagePartClass *klass)
 	
 	gobject_class->finalize = g_mime_message_part_finalize;
 	
-	object_class->init = message_part_init;
 	object_class->prepend_header = message_part_prepend_header;
 	object_class->append_header = message_part_append_header;
 	object_class->remove_header = message_part_remove_header;
@@ -124,14 +122,6 @@ g_mime_message_part_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-
-static void
-message_part_init (GMimeObject *object)
-{
-	/* no-op */
-	GMIME_OBJECT_CLASS (parent_class)->init (object);
-}
-
 static void
 message_part_prepend_header (GMimeObject *object, const char *header, const char *value)
 {
