@@ -53,7 +53,6 @@ static void g_mime_multipart_init (GMimeMultipart *multipart, GMimeMultipartClas
 static void g_mime_multipart_finalize (GObject *object);
 
 /* GMimeObject class methods */
-static void multipart_init (GMimeObject *object);
 static void multipart_prepend_header (GMimeObject *object, const char *header, const char *value);
 static void multipart_append_header (GMimeObject *object, const char *header, const char *value);
 static void multipart_set_header (GMimeObject *object, const char *header, const char *value);
@@ -66,7 +65,7 @@ static ssize_t multipart_write_to_stream (GMimeObject *object, GMimeStream *stre
 /* GMimeMultipart class methods */
 static void multipart_add_part (GMimeMultipart *multipart, GMimeObject *part);
 static void multipart_add_part_at (GMimeMultipart *multipart, GMimeObject *part, int index);
-static void multipart_remove_part (GMimeMultipart *multipart, GMimeObject *part);
+static gboolean multipart_remove_part (GMimeMultipart *multipart, GMimeObject *part);
 static GMimeObject *multipart_remove_part_at (GMimeMultipart *multipart, int index);
 static GMimeObject *multipart_get_part (GMimeMultipart *multipart, int index);
 static int multipart_get_number (GMimeMultipart *multipart);
@@ -112,7 +111,6 @@ g_mime_multipart_class_init (GMimeMultipartClass *klass)
 	
 	gobject_class->finalize = g_mime_multipart_finalize;
 	
-	object_class->init = multipart_init;
 	object_class->prepend_header = multipart_prepend_header;
 	object_class->append_header = multipart_append_header;
 	object_class->remove_header = multipart_remove_header;
@@ -135,43 +133,30 @@ g_mime_multipart_class_init (GMimeMultipartClass *klass)
 static void
 g_mime_multipart_init (GMimeMultipart *multipart, GMimeMultipartClass *klass)
 {
+	multipart->children = g_ptr_array_new ();
 	multipart->boundary = NULL;
 	multipart->preface = NULL;
 	multipart->postface = NULL;
-	multipart->subparts = NULL;
 }
 
 static void
 g_mime_multipart_finalize (GObject *object)
 {
 	GMimeMultipart *multipart = (GMimeMultipart *) object;
-	GList *node;
+	guint i;
 	
 	g_free (multipart->boundary);
 	g_free (multipart->preface);
 	g_free (multipart->postface);
 	
-	node = multipart->subparts;
-	while (node) {
-		GMimeObject *part;
-		
-		part = node->data;
-		g_object_unref (part);
-		node = node->next;
-	}
-	g_list_free (multipart->subparts);
+	for (i = 0; i < multipart->children->len; i++)
+		g_object_unref (multipart->children->pdata[i]);
+	
+	g_ptr_array_free (multipart->children, TRUE);
 	
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-
-static void
-multipart_init (GMimeObject *object)
-{
-	/* no-op */
-	GMIME_OBJECT_CLASS (parent_class)->init (object);
-}
-
 static void
 multipart_prepend_header (GMimeObject *object, const char *header, const char *value)
 {
@@ -250,7 +235,7 @@ multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
 	GMimeMultipart *multipart = (GMimeMultipart *) object;
 	ssize_t nwritten, total = 0;
 	GMimeObject *part;
-	GList *node;
+	guint i;
 	
 	/* make sure a boundary is set unless we are writing out a raw
 	 * header (in which case it should already be set... or if
@@ -279,9 +264,8 @@ multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
 		total += nwritten;
 	}
 	
-	node = multipart->subparts;
-	while (node) {
-		part = node->data;
+	for (i = 0; i < multipart->children->len; i++) {
+		part = multipart->children->pdata[i];
 		
 		/* write the boundary */
 		if ((nwritten = g_mime_stream_printf (stream, "\n--%s\n", multipart->boundary)) == -1)
@@ -294,8 +278,6 @@ multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
 			return -1;
 		
 		total += nwritten;
-		
-		node = node->next;
 	}
 	
 	/* write the end-boundary (but only if a boundary is set) */
@@ -313,7 +295,7 @@ multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
 		
 		total += nwritten;
 	}
-		
+	
 	return total;
 }
 
@@ -439,7 +421,8 @@ static void
 multipart_add_part (GMimeMultipart *multipart, GMimeObject *part)
 {
 	g_object_ref (part);
-	multipart->subparts = g_list_append (multipart->subparts, part);
+	
+	g_ptr_array_add (multipart->children, part);
 }
 
 
@@ -460,11 +443,32 @@ g_mime_multipart_add_part (GMimeMultipart *multipart, GMimeObject *part)
 }
 
 
+static void
+ptr_array_insert (GPtrArray *array, guint index, gpointer object)
+{
+	unsigned char *dest, *src;
+	guint n;
+	
+	g_ptr_array_set_size (array, array->len + 1);
+	
+	if (index == array->len) {
+		/* need to move items down */
+		dest = ((unsigned char *) array->pdata) + (sizeof (void *) * (index + 1));
+		src = ((unsigned char *) array->pdata) + (sizeof (void *) * index);
+		n = array->len - index - 1;
+		
+		g_memmove (dest, src, (sizeof (void *) * n));
+	}
+	
+	array->pdata[index] = object;
+}
+
 static void
 multipart_add_part_at (GMimeMultipart *multipart, GMimeObject *part, int index)
 {
 	g_object_ref (part);
-	multipart->subparts = g_list_insert (multipart->subparts, part, index);
+	
+	ptr_array_insert (multipart->children, index, part);
 }
 
 
@@ -487,39 +491,15 @@ g_mime_multipart_add_part_at (GMimeMultipart *multipart, GMimeObject *part, int
 }
 
 
-static void
+static gboolean
 multipart_remove_part (GMimeMultipart *multipart, GMimeObject *part)
 {
-	GList *node;
-	
-	/* *sigh* fucking glist... */
-	
-	node = multipart->subparts;
-	while (node) {
-		if (node->data == (gpointer) part)
-			break;
-		node = node->next;
-	}
-	
-	if (node == NULL) {
-		d(g_warning ("multipart_remove_part: %p does not seem to be a subpart of %p",
-			     part, multipart));
-		return;
-	}
-	
-	if (node == multipart->subparts) {
-		if (node->next)
-			node->next->prev = NULL;
-		multipart->subparts = node->next;
-	} else {
-		if (node->next)
-			node->next->prev = node->prev;
-		node->prev->next = node->next;
-	}
-	
-	g_list_free_1 (node);
+	if (!g_ptr_array_remove (multipart->children, part))
+		return FALSE;
 	
 	g_object_unref (part);
+	
+	return TRUE;
 }
 
 
@@ -529,14 +509,16 @@ multipart_remove_part (GMimeMultipart *multipart, GMimeObject *part)
  * @part: mime part
  *
  * Removes the specified mime part from the multipart.
+ *
+ * Returns: %TRUE if the part was removed or %FALSE otherwise.
  **/
-void
+gboolean
 g_mime_multipart_remove_part (GMimeMultipart *multipart, GMimeObject *part)
 {
-	g_return_if_fail (GMIME_IS_MULTIPART (multipart));
-	g_return_if_fail (GMIME_IS_OBJECT (part));
+	g_return_val_if_fail (GMIME_IS_MULTIPART (multipart), FALSE);
+	g_return_val_if_fail (GMIME_IS_OBJECT (part), FALSE);
 	
-	GMIME_MULTIPART_GET_CLASS (multipart)->remove_part (multipart, part);
+	return GMIME_MULTIPART_GET_CLASS (multipart)->remove_part (multipart, part);
 }
 
 
@@ -544,26 +526,13 @@ static GMimeObject *
 multipart_remove_part_at (GMimeMultipart *multipart, int index)
 {
 	GMimeObject *part;
-	GList *node;
 	
-	if (!(node = g_list_nth (multipart->subparts, index))) {
-		d(g_warning ("multipart_remove_part_at: no part at index %d within %p", index, multipart));
+	if (index >= multipart->children->len)
 		return NULL;
-	}
 	
-	part = node->data;
+	part = multipart->children->pdata[index];
 	
-	if (node == multipart->subparts) {
-		if (node->next)
-			node->next->prev = NULL;
-		multipart->subparts = node->next;
-	} else {
-		if (node->next)
-			node->next->prev = node->prev;
-		node->prev->next = node->next;
-	}
-	
-	g_list_free_1 (node);
+	g_ptr_array_remove_index (multipart->children, index);
 	
 	return part;
 }
@@ -592,15 +561,11 @@ static GMimeObject *
 multipart_get_part (GMimeMultipart *multipart, int index)
 {
 	GMimeObject *part;
-	GList *node;
 	
-	if (!(node = g_list_nth (multipart->subparts, index))) {
-		d(g_warning ("multipart_get_part: no part at index %d within %p", index, multipart));
+	if (index >= multipart->children->len)
 		return NULL;
-	}
-	
-	part = node->data;
 	
+	part = multipart->children->pdata[index];
 	g_object_ref (part);
 	
 	return part;
@@ -629,10 +594,7 @@ g_mime_multipart_get_part (GMimeMultipart *multipart, int index)
 static int
 multipart_get_number (GMimeMultipart *multipart)
 {
-	if (!multipart->subparts)
-		return 0;
-	
-	return g_list_length (multipart->subparts);
+	return multipart->children->len;
 }
 
 
@@ -764,18 +726,15 @@ g_mime_multipart_get_boundary (GMimeMultipart *multipart)
 void
 g_mime_multipart_foreach (GMimeMultipart *multipart, GMimePartFunc callback, gpointer user_data)
 {
-	GList *subpart;
+	GMimeObject *part;
+	guint i;
 	
 	g_return_if_fail (GMIME_IS_MULTIPART (multipart));
 	g_return_if_fail (callback != NULL);
 	
-	subpart = multipart->subparts;
-	while (subpart) {
-		GMimeObject *part = subpart->data;
-		
+	for (i = 0; i < multipart->children->len; i++) {
+		part = multipart->children->pdata[i];
 		callback (part, user_data);
-		
-		subpart = subpart->next;
 	}
 }
 
@@ -795,7 +754,9 @@ GMimeObject *
 g_mime_multipart_get_subpart_from_content_id (GMimeMultipart *multipart, const char *content_id)
 {
 	GMimeObject *object = (GMimeObject *) multipart;
-	GList *subparts;
+	GMimeObject *subpart, *part;
+	GMimeMultipart *mpart;
+	guint i;
 	
 	g_return_val_if_fail (GMIME_IS_MULTIPART (multipart), NULL);
 	g_return_val_if_fail (content_id != NULL, NULL);
@@ -805,24 +766,17 @@ g_mime_multipart_get_subpart_from_content_id (GMimeMultipart *multipart, const c
 		return object;
 	}
 	
-	subparts = multipart->subparts;
-	while (subparts) {
-		GMimeObject *part = NULL;
-		GMimeObject *subpart;
-		
-		subpart = subparts->data;
+	for (i = 0; i < multipart->children->len; i++) {
+		subpart = multipart->children->pdata[i];
 		
 		if (GMIME_IS_MULTIPART (subpart)) {
-			part = g_mime_multipart_get_subpart_from_content_id (GMIME_MULTIPART (subpart), content_id);
+			mpart = (GMimeMultipart *) subpart;
+			if ((part = g_mime_multipart_get_subpart_from_content_id (mpart, content_id)))
+				return part;
 		} else if (subpart->content_id && !strcmp (subpart->content_id, content_id)) {
 			g_object_ref (subpart);
-			part = subpart;
+			return subpart;
 		}
-		
-		if (part)
-			return part;
-		
-		subparts = subparts->next;
 	}
 	
 	return NULL;
