@@ -41,18 +41,22 @@ void
 print_mime_struct (GMimeObject *part, int depth)
 {
 	const GMimeContentType *type;
+	GMimeMultipart *multipart;
+	GMimeObject *subpart;
+	int i, n;
 	
 	print_depth (depth);
 	type = g_mime_object_get_content_type (part);
 	fprintf (stdout, "Content-Type: %s/%s\n", type->type, type->subtype);
 		
 	if (GMIME_IS_MULTIPART (part)) {
-		GList *subpart;
+		multipart = (GMimeMultipart *) part;
 		
-		subpart = GMIME_MULTIPART (part)->subparts;
-		while (subpart) {
-			print_mime_struct (subpart->data, depth + 1);
-			subpart = subpart->next;
+		n = g_mime_multipart_get_number (multipart);
+		for (i = 0; i < n; i++) {
+			subpart = g_mime_multipart_get_part (multipart, i);
+			print_mime_struct (subpart, depth + 1);
+			g_object_unref (subpart);
 		}
 	}
 }
@@ -80,7 +84,7 @@ int main (int argc, char **argv)
 		
 		stream = g_mime_stream_fs_new (fd);
 		g_mime_parser_init_with_stream (parser, stream);
-		g_mime_stream_unref (stream);
+		g_object_unref (stream);
 		
 		if (!(message = g_mime_parser_construct_message (parser)))
 			return -2;
@@ -89,8 +93,8 @@ int main (int argc, char **argv)
 			return -3;
 		
 		g_ptr_array_add (partials, message->mime_part);
-		g_mime_object_ref (GMIME_OBJECT (message->mime_part));
-		g_mime_object_unref (GMIME_OBJECT (message));
+		g_object_ref (message->mime_part);
+		g_object_unref (message);
 	}
 	
 	g_object_unref (parser);
@@ -106,8 +110,8 @@ int main (int argc, char **argv)
 	
 	print_mime_struct (message->mime_part, 0);
 	
-	g_mime_object_unref (GMIME_OBJECT (message));
-	g_mime_stream_unref (stream);
+	g_object_unref (message);
+	g_object_unref (stream);
 	
 	g_mime_shutdown ();
 	
